from typing import List

import numpy as np
from gbvision.constants.types import Frame, RotatedRect, Number, Point, FilterFunction

from gbvision.constants.system import EMPTY_PIPELINE
from gbvision.models.contours import find_contours, FilterContours, sort_contours, contours_to_rotated_rects_sorted
from gbvision.models.shapes import filter_inner_rotated_rects
from .object_finder import ObjectFinder


class RotatedRectFinder(ObjectFinder):
    """
    finds a rectangular object, but rotated. recommended to use when you know the shape isn't parallel to the camera

    :param threshold_func: a pipeline (or any sort of function) that returns a binary threshold of the object
     the finder is searching, the object needs to be white and the rest if the image black (doesn't
     have to be perfect)
    :param area_scalar: optional, a scalar to multiply the area by, for fine tuning of the function's output
    :param contour_min_area: the minimal area of a contour, used for FilterContours, default is 0 (no area limit)
    """

    def __init__(self, threshold_func: FilterFunction, game_object, area_scalar=1.0, contour_min_area=0.0):
        ObjectFinder.__init__(self, game_object, area_scalar=area_scalar)
        self._full_pipeline = (EMPTY_PIPELINE +
                               threshold_func +
                               find_contours +
                               FilterContours(min_area=contour_min_area) +
                               sort_contours +
                               contours_to_rotated_rects_sorted +
                               filter_inner_rotated_rects)

    def find_shapes(self, frame: Frame) -> List[RotatedRect]:
        return self._full_pipeline(frame)

    @staticmethod
    def _shape_root_area(shape: RotatedRect) -> Number:
        return np.sqrt(shape[1][0] * shape[1][1])

    @staticmethod
    def _shape_center(shape: RotatedRect) -> Point:
        return shape[0]



from copy import deepcopy

from gbvision.models.shapes import rect_collision
from gbvision.constants.types import Rect, Number, Point
from gbvision.continuity.continues_shape import ContinuesShape


class ContinuesRect(ContinuesShape):
    """
    An implementation of ContinuesShape to rectangles.
    used to try and check whether two rectangles are indeed the same one.
    """

    def __init__(self, shape: Rect, *args, **kwargs):
        ContinuesShape.__init__(self, shape=shape, *args, **kwargs)

    def _shape_collision(self, shape) -> bool:
        return rect_collision(self._shape, shape)

    @staticmethod
    def _shape_area(rect: Rect) -> Number:
        return rect[2] * rect[3]

    @staticmethod
    def _from_bounding_rect(bounding_rect: Rect):
        return deepcopy(bounding_rect)

    @staticmethod
    def _to_bounding_rect(rect: Rect) -> Rect:
        return deepcopy(rect)

    @staticmethod
    def _shape_center(shape: Rect) -> Point:
        center_x = shape[0] + shape[2] / 2
        center_y = shape[1] + shape[3] / 2
        return center_x, center_y



# cameras
from .cameras.camera import Camera
from .cameras.stream_camera import StreamCamera
from .cameras.camera_data import CameraData
from .cameras.usb_camera import USBCamera
from .cameras.camera_list import CameraList
from .cameras.usb_stream_camera import USBStreamCamera
from .cameras.async_camera import AsyncCamera
from .cameras.async_usb_camera import AsyncUSBCamera

# constants
from .constants.cameras import LIFECAM_3000, LIFECAM_STUDIO, UNKNOWN_CAMERA
from .constants.math import EPSILON, SQRT_PI
from .constants.system import EMPTY_PIPELINE
from .constants.types import RotatedRect, Frame, Color, Circle, Contour, Rect, FixedPolygon, Ellipse, Polygon, Point, \
    Location, FilterFunction, Coordinates, Number, ROI, Shape

# continuity
from .continuity.continues_circle import ContinuesCircle
from .continuity.continues_rect import ContinuesRect
from .continuity.continues_rotated_rect import ContinuesRotatedRect
from .continuity.continues_shape import ContinuesShape
from .continuity.continues_shape_wrapper import ContinuesShapeWrapper

# exceptions
from .exceptions.vision_warning import VisionWarning
from .exceptions.vision_exception import VisionException
from .exceptions.could_not_read_frame_exception import CouldNotReadFrameException
from .exceptions.device_not_found_exception import DeviceNotFoundException
from .exceptions.stream_closed import StreamClosed
from .exceptions.tcp_stream_closed import TCPStreamClosed

# finders
from .finders.object_finder import ObjectFinder
from .finders.circle_finder import CircleFinder
from .finders.rect_finder import RectFinder
from .finders.polygon_finder import PolygonFinder
from .finders.rotated_rect_finder import RotatedRectFinder
from .finders.target_pair_finder import TargetPairFinder

# gui
from .gui.window import Window
from .gui.stream_window import StreamWindow
from .gui.feed_window import FeedWindow
from .gui.camera_window import CameraWindow
from .gui.recording_camera_window import RecordingCameraWindow
from .gui.recording_stream_window import RecordingStreamWindow
from .gui.drawing_tools import DrawCircles, DrawContours, DrawEllipses, DrawRects, DrawRotatedRects
from .gui.recording_feed_window import RecordingFeedWindow
from .gui.drawing_functions import draw_ellipses, draw_rotated_rects, draw_rects, draw_circles, draw_contours, draw_text
from .gui.recording_wrapper_opencv_window import RecordingWrapperOpenCVWindow
from .gui.recording_opencv_window import RecordingOpenCVWindow
from .gui.recording_window import RecordingWindow
from .gui.wrapper_opencv_window import WrapperOpenCVWindow
from .gui.wrapper_window import WrapperWindow
from .gui.opencv_window import OpenCVWindow

# models
from .models.basic_ops import corners, edges, sharpen, blur, blue, red, green, gray
from .models.contours import contour_center, contours_centers, contours_to_circles, contours_to_circles_sorted, \
    contours_to_ellipses, contours_to_ellipses_sorted, contours_to_polygons, contours_to_rects, \
    contours_to_rects_sorted, contours_to_rotated_rects, contours_to_rotated_rects_sorted, FilterContours, \
    find_contours, sort_contours, sort_rects, sort_circles, sort_rotated_rects, sort_ellipses
from .models.shapes import circle_collision, filter_inner_circles, filter_inner_rects, rect_collision, \
    rotated_rect_collision, convex_shape_collision, filter_inner_convex_shapes, filter_inner_rotated_rects
from .models.denoising import Dilate, MedianBlur, Erode

# net
from .net.stream_receiver import StreamReceiver
from .net.stream_broadcaster import StreamBroadcaster
from .net.tcp_stream_broadcaster import TCPStreamBroadcaster
from .net.tcp_stream_receiver import TCPStreamReceiver
from .net.udp_stream_broadcaster import UDPStreamBroadcaster
from .net.udp_stream_receiver import UDPStreamReceiver
from .net.async_tcp_stream_receiver import AsyncTCPStreamReceiver
from .net.async_udp_stream_receiver import AsyncUDPStreamReceiver

# thresholds
from .thresholds.color_threshold import ColorThreshold
from .thresholds.threshold import Threshold, ThresholdGroup

# tools
from .tools.list_tools import split_list
from .tools.image_tools import crop, median_threshold
from .tools.finding_tools import distance_from_object, plane_angle_by_location, plane_distance_from_object

# utils
from .utils.pipeline import PipeLine
from .utils.game_object import GameObject
from .utils.tracker import Tracker


# configure opencv
def cv_config():
    """
    configure some opencv stuff so that it doesn't cause problems
    called automatically when importing gbvision, and does not need to be called by the user
    """
    import cv2
    if cv2.__version__[0] == '2':
        for i in filter(lambda attr: attr.startswith("CV_CAP_PROP"), dir(cv2.cv)):
            object.__setattr__(cv2, i[3:], cv2.cv.__getattribute__(i))
            cv2.__dict__[i[3:]] = cv2.cv.__dict__[i]


cv_config()
del cv_config
